package example.pocketbase

import upickle.default._
import Api.Error
import Models._
import requests.Response

final case class Api(pocketbaseUrl: String, usersCollection: String = "users") {
  def refreshSession(jwt: String): Either[Error, AuthReply] = {
    val path = s"/api/collections/${usersCollection}/auth-refresh"

    val refreshResult = requests.post(
      url = pocketbaseUrl ++ path,
      headers = List("Authorization" -> jwt)
    )

    refreshResult.statusCode match {
      case 200 =>
        val newAuth = read[AuthReply](refreshResult.text())
        Right(newAuth)
      case 404 => Left(Error("not found"))
      case _ => Left(Error("other error"))
    }
  }

  def listAuthMethods(): AvailableAuthMethods = {
    val path = s"/api/collections/$usersCollection/auth-methods"

    val availableAuth = requests.get(
      url = pocketbaseUrl ++ path
    )

    read[AvailableAuthMethods](availableAuth.text())
  }

  def authWithOauth(
      provider: String,
      code: String,
      verifier: String,
      redirectUrl: String
  ): Either[Error, AuthReply] = {
    val path = s"/api/collections/${usersCollection}/auth-with-oauth2"

    val authResult: Response = requests.post(
      url = pocketbaseUrl ++ path,
      data = Map(
        "provider" -> provider,
        "code" -> code,
        "codeVerifier" -> verifier,
        "redirectUrl" -> redirectUrl
      )
    )

    authResult.statusCode match {
      case 200 =>
        val result = read[AuthReply](authResult.text())
        Right(result)
      case _ => Left(Error("auth not successful"))
    }
  }
}

object Api {
  final case class Error(msg: String)
}
